@@ -1,4 +1,7 @@
-mod registration;
+mod registration {
+    include!(concat!(env!("OUT_DIR"), "/registration.rs"));
+}
+
 mod timers;
 
 use ndarray::prelude::*;
@@ -8,29 +11,42 @@ use rand_pcg::Pcg64Mcg;
 use rayon::prelude::*;
 use rayon::ThreadPool;
 use roxido::*;
-use std::convert::TryInto;
 use std::path::Path;
 use timers::{EchoTimer, PeriodicTimer};
 
 #[allow(unused_imports)]
 use approx::assert_ulps_eq;
 
+fn get(samples: RList, index: usize) -> RMatrix {
+    match samples.get(index) {
+        Ok(element) => {
+            let matrix = element.as_matrix_or_stop("All elements of 'samples' must be a matrix.");
+            if !matrix.is_double() {
+                stop!("All elements of 'samples' mus be of storage mode 'double'");
+            }
+            matrix
+        }
+        Err(_) => stop!("Index into 'samples' is out of bounds."),
+    }
+}
+
 #[roxido]
 fn fangs(
-    samples: Rval,
-    n_iterations: Rval,
-    max_seconds: Rval,
-    n_baselines: Rval,
-    n_sweet: Rval,
-    a: Rval,
-    n_cores: Rval,
-    use_neighbors: Rval,
-    quiet: Rval,
-) -> Rval {
+    samples: RObject,
+    n_iterations: RObject,
+    max_seconds: RObject,
+    n_baselines: RObject,
+    n_sweet: RObject,
+    a: RObject,
+    n_cores: RObject,
+    use_neighbors: RObject,
+    quiet: RObject,
+) -> RObject {
     let mut timer = EchoTimer::new();
+    let samples = samples.as_list_or_stop("'samples' must be a list.");
     let n_samples = samples.len();
     if n_samples < 1 {
-        panic!("Number of samples must be at least one.");
+        stop!("Number of samples must be at least one.");
     }
     let a = a.as_f64();
     let threshold = a / 2.0;
@@ -51,13 +67,10 @@ fn fangs(
             .unwrap_or_default()
             .join("FANGS_STATUS"),
     };
-    let o = samples.get_list_element(0);
-    if !o.is_double() || !o.is_matrix() {
-        panic!("All elements of 'samples' must be double matrices.");
-    }
+    let o = get(samples, 0);
     let n_items = o.nrow();
     let mut max_n_features_observed = 0;
-    let mut rng = Pcg64Mcg::from_seed(r::random_bytes::<16>());
+    let mut rng = Pcg64Mcg::from_seed(R::random_bytes::<16>());
     let mut interrupted = false;
     if timer.echo() {
         interrupted |= rprint!(
@@ -73,13 +86,13 @@ fn fangs(
                 .unwrap()
                 .as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let mut views = Vec::with_capacity(n_samples);
     for i in 0..n_samples {
-        let o = samples.get_list_element(i);
-        if !o.is_double() || !o.is_matrix() || o.nrow() != n_items {
-            panic!("All elements of 'samples' must be double matrices with a consistent number of rows.");
+        let o = get(samples, i);
+        if o.nrow() != n_items {
+            stop!("All elements of 'samples' must have the same number of rows.");
         }
         let view = make_view(o);
         max_n_features_observed = max_n_features_observed.max(view.ncols());
@@ -90,7 +103,7 @@ fn fangs(
             "{}",
             timer.stamp("Made data structures.\n").unwrap().as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let baselines_with_rngs: Vec<_> = rand::seq::index::sample(&mut rng, n_samples, n_baselines)
         .into_iter()
@@ -110,20 +123,18 @@ fn fangs(
                     .map(|zz| {
                         let weight_matrix = make_weight_matrix(view, *zz, a).unwrap();
                         let solution = lapjv::lapjv(&weight_matrix).unwrap();
-                        let aligned =
-                            Array2::from_shape_fn((n_items, max_n_features_observed), |(i, j)| {
-                                if j >= solution.0.len() {
+                        Array2::from_shape_fn((n_items, max_n_features_observed), |(i, j)| {
+                            if j >= solution.0.len() {
+                                0.0
+                            } else {
+                                let jj = solution.0[j];
+                                if jj >= zz.ncols() {
                                     0.0
                                 } else {
-                                    let jj = solution.0[j];
-                                    if jj >= zz.ncols() {
-                                        0.0
-                                    } else {
-                                        zz[[i, jj]]
-                                    }
+                                    zz[[i, jj]]
                                 }
-                            });
-                        aligned
+                            }
+                        })
                     })
                     .reduce(
                         || Array2::zeros((n_items, max_n_features_observed)),
@@ -133,12 +144,14 @@ fn fangs(
                 let initial_estimate_with_zero_columns =
                     elementwise_means.mapv(|x| if x < threshold { 0.0 } else { 1.0 });
                 let mut which: Vec<usize> = Vec::new();
-                let mut column_counter = 0;
-                for column in initial_estimate_with_zero_columns.columns() {
+                for (column_counter, column) in initial_estimate_with_zero_columns
+                    .columns()
+                    .into_iter()
+                    .enumerate()
+                {
                     if column.iter().any(|&x| x > 0.0) {
                         which.push(column_counter)
                     }
-                    column_counter += 1;
                 }
                 let initial_estimate = if which.is_empty() {
                     Array2::zeros((n_items, 1))
@@ -158,12 +171,12 @@ fn fangs(
             "{}",
             timer.stamp("Made initial estimates.\n").unwrap().as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let mut initials = Vec::with_capacity(initials_with_rngs.len());
     for (z, rng) in initials_with_rngs {
-        if interrupted || r::check_user_interrupt() {
-            panic!("Caught user interrupt before main loop, so aborting.");
+        if interrupted || R::check_user_interrupt() {
+            stop!("Caught user interrupt before main loop, so aborting.");
         }
         let weight_matrices = make_weight_matrices(z.view(), &views, a, &pool);
         let loss = expected_loss_from_weight_matrices(&weight_matrices[..], &pool);
@@ -177,7 +190,7 @@ fn fangs(
                 .unwrap()
                 .as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     initials.sort_unstable_by(|x, y| x.1.partial_cmp(&y.1).unwrap());
     initials.truncate(n_sweet);
@@ -200,7 +213,7 @@ fn fangs(
                 .unwrap()
                 .as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let seconds_in_initialization = timer.total_as_secs_f64();
     let mut period_timer = PeriodicTimer::new(1.0);
@@ -241,7 +254,7 @@ fn fangs(
                         let total_length = n_items * n_features;
                         let index = index_1d_to_2d(rng.gen_range(0..total_length), n_features);
                         flip_bit(z, weight_matrices, a, index, &views);
-                        let new_loss = expected_loss_from_weight_matrices(&weight_matrices, &pool);
+                        let new_loss = expected_loss_from_weight_matrices(weight_matrices, &pool);
                         if new_loss < *loss {
                             *n_accepts += 1;
                             *when = iteration_counter;
@@ -263,7 +276,7 @@ fn fangs(
                             )
                             .as_str()
                         );
-                        r::flush_console();
+                        R::flush_console();
                     }
                     sweets.sort_unstable_by(|x, y| x.1.partial_cmp(&y.1).unwrap());
                     let best = sweets.first().unwrap();
@@ -280,19 +293,19 @@ fn fangs(
                     )
                         .as_str()
                     );
-                    r::flush_console();
+                    R::flush_console();
                 });
             }
-            if interrupted || r::check_user_interrupt() {
+            if interrupted || R::check_user_interrupt() {
                 rprint!("\nCaught user interrupt, so breaking out early.");
-                r::flush_console();
+                R::flush_console();
                 break;
             }
         }
     }
     if !quiet {
         rprint!("\n");
-        r::flush_console();
+        R::flush_console();
     }
     let seconds_in_sweetening = timer.total_as_secs_f64() - seconds_in_initialization;
     if timer.echo() {
@@ -303,7 +316,7 @@ fn fangs(
                 .unwrap()
                 .as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     sweets.sort_unstable_by(|x, y| x.1.partial_cmp(&y.1).unwrap());
     let (best_z, best_loss, _, sweeten_number, n_accepts, best_iteration, _) =
@@ -321,7 +334,7 @@ fn fangs(
             )
             .as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let columns_to_keep: Vec<usize> = best_z
         .axis_iter(Axis(1))
@@ -334,15 +347,15 @@ fn fangs(
             }
         })
         .collect();
-    let (estimate, estimate_slice) = Rval::new_matrix_double(n_items, columns_to_keep.len(), pc);
+    let (estimate, estimate_slice) = RMatrix::new_double(n_items, columns_to_keep.len(), pc);
     columns_to_keep
         .iter()
         .enumerate()
         .for_each(|(j_new, j_old)| {
             matrix_copy_into_column(estimate_slice, n_items, j_new, best_z.column(*j_old).iter())
         });
-    let list = Rval::new_list(8, pc);
-    list.names_gets(rval!([
+    let list = RList::new(8, pc);
+    let _ = list.names_gets(rstr!([
         "estimate",
         "expectedLoss",
         "iteration",
@@ -352,33 +365,35 @@ fn fangs(
         "secondsTotal",
         "whichSweet",
     ]));
-    list.set_list_element(0, estimate);
-    list.set_list_element(1, rval!(best_loss));
-    list.set_list_element(2, rval!(best_iteration as i32));
-    list.set_list_element(3, rval!((iteration_counter + 1) as i32));
-    list.set_list_element(4, rval!(seconds_in_initialization));
-    list.set_list_element(5, rval!(seconds_in_sweetening));
-    list.set_list_element(7, rval!((sweeten_number + 1) as i32));
-    list.set_list_element(6, rval!(timer.total_as_secs_f64()));
+    let _ = list.set(0, estimate);
+    let _ = list.set(1, rvec!(best_loss));
+    let _ = list.set(2, rvec!(best_iteration as i32));
+    let _ = list.set(3, rvec!((iteration_counter + 1) as i32));
+    let _ = list.set(4, rvec!(seconds_in_initialization));
+    let _ = list.set(5, rvec!(seconds_in_sweetening));
+    let _ = list.set(7, rvec!((sweeten_number + 1) as i32));
+    let _ = list.set(6, rvec!(timer.total_as_secs_f64()));
     if timer.echo() {
         rprint!("{}", timer.stamp("Finalized results.\n").unwrap().as_str());
-        r::flush_console();
+        R::flush_console();
     }
     list
 }
 
 #[roxido]
-fn fangs_double_greedy(samples: Rval, max_seconds: Rval, a: Rval, n_cores: Rval) -> Rval {
+fn fangs_double_greedy(
+    samples: RObject,
+    max_seconds: RObject,
+    a: RObject,
+    n_cores: RObject,
+) -> RObject {
     let timer = EchoTimer::new();
-    let o = samples.get_list_element(0);
-    if !o.is_double() || !o.is_matrix() {
-        panic!("All elements of 'samples' must be double matrices.");
-    }
-    let n_items = o.nrow();
+    let samples = samples.as_list_or_stop("'samples' must be a list.");
     let n_samples = samples.len();
     if n_samples < 1 {
-        panic!("Number of samples must be at least one.");
+        stop!("Number of samples must be at least one.");
     }
+    let n_items = get(samples, 0).nrow();
     let max_seconds = max_seconds.as_f64();
     let a = a.as_f64();
     let n_cores = n_cores.as_usize();
@@ -389,9 +404,9 @@ fn fangs_double_greedy(samples: Rval, max_seconds: Rval, a: Rval, n_cores: Rval)
     let mut max_n_features_observed = 0;
     let mut views = Vec::with_capacity(n_samples);
     for i in 0..n_samples {
-        let o = samples.get_list_element(i);
-        if !o.is_double() || !o.is_matrix() || o.nrow() != n_items {
-            panic!("All elements of 'samples' must be double matrices with a consistent number of rows.");
+        let o = get(samples, i);
+        if o.nrow() != n_items {
+            stop!("All elements of 'samples' must have the same number of rows.");
         }
         let view = make_view(o);
         max_n_features_observed = max_n_features_observed.max(view.ncols());
@@ -409,7 +424,7 @@ fn fangs_double_greedy(samples: Rval, max_seconds: Rval, a: Rval, n_cores: Rval)
         max_seconds,
         &timer,
     );
-    let (estimate, estimate_slice) = Rval::new_matrix_double(n_items, z.ncols(), pc);
+    let (estimate, estimate_slice) = RMatrix::new_double(n_items, z.ncols(), pc);
     let mut index = 0;
     for j in 0..z.ncols() {
         for i in 0..n_items {
@@ -417,11 +432,11 @@ fn fangs_double_greedy(samples: Rval, max_seconds: Rval, a: Rval, n_cores: Rval)
             index += 1;
         }
     }
-    let list = Rval::new_list(3, pc);
-    list.names_gets(rval!(["estimate", "expectedLoss", "secondsTotal",]));
-    list.set_list_element(0, estimate);
-    list.set_list_element(1, rval!(loss));
-    list.set_list_element(2, rval!(timer.total_as_secs_f64()));
+    let list = RList::new(3, pc);
+    let _ = list.names_gets(rstr!(["estimate", "expectedLoss", "secondsTotal",]));
+    let _ = list.set(0, estimate);
+    let _ = list.set(1, rvec!(loss));
+    let _ = list.set(2, rvec!(timer.total_as_secs_f64()));
     list
 }
 
@@ -435,7 +450,7 @@ fn neighborhood_sweeten(
     max_seconds: f64,
     timer: &EchoTimer,
 ) -> f64 {
-    let mut outer_loss = expected_loss_from_weight_matrices(&weight_matrices[..], pool);
+    let mut outer_loss = expected_loss_from_weight_matrices(weight_matrices, pool);
     loop {
         if timer.echo() {
             println!("Current loss: {}", outer_loss);
@@ -449,12 +464,12 @@ fn neighborhood_sweeten(
         for i in 0..n_items {
             for j in 0..z.ncols() {
                 let candidate_loss = expected_loss_from_weight_matrices_if_flip_bit(
-                    &z,
+                    z,
                     weight_matrices,
                     a,
                     [i, j],
-                    &views,
-                    &pool,
+                    views,
+                    pool,
                 );
                 if candidate_loss < best_candidate_loss {
                     best_index = [i, j];
@@ -463,7 +478,7 @@ fn neighborhood_sweeten(
             }
         }
         if best_candidate_loss < outer_loss {
-            flip_bit(z, weight_matrices, a, best_index, &views);
+            flip_bit(z, weight_matrices, a, best_index, views);
             outer_loss = best_candidate_loss;
         } else {
             break;
@@ -473,11 +488,12 @@ fn neighborhood_sweeten(
 }
 
 #[roxido]
-fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
+fn draws(samples: RObject, a: RObject, n_cores: RObject, quiet: RObject) -> RObject {
     let mut timer = EchoTimer::new();
+    let samples = samples.as_list_or_stop("Expected a list.");
     let n_samples = samples.len();
     if n_samples < 1 {
-        panic!("Number of samples must be at least one.");
+        stop!("Number of samples must be at least one.");
     }
     let a = a.as_f64();
     //let n_candidates = n_candidates.as_usize().max(1).min(n_samples);
@@ -489,12 +505,8 @@ fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
         .build()
         .unwrap();
     let quiet = quiet.as_bool();
-    let o = samples.get_list_element(0);
-    if !o.is_double() || !o.is_matrix() {
-        panic!("All elements of 'samples' must be double matrices.");
-    }
-    let n_items = o.nrow();
-    let mut rng = Pcg64Mcg::from_seed(r::random_bytes::<16>());
+    let n_items = get(samples, 0).nrow();
+    let mut rng = Pcg64Mcg::from_seed(R::random_bytes::<16>());
     let mut interrupted = false;
     if timer.echo() {
         interrupted |= rprint!(
@@ -510,15 +522,15 @@ fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
                 .unwrap()
                 .as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let mut views = Vec::with_capacity(n_samples);
     for i in 0..n_samples {
-        let o = samples.get_list_element(i);
-        if !o.is_double() || !o.is_matrix() || o.nrow() != n_items {
-            panic!("All elements of 'samples' must be double matrices with a consistent number of rows.");
+        let o = get(samples, i);
+        if o.nrow() != n_items {
+            stop!("All elements of 'samples' must have the same number of rows.");
         }
-        let view = make_view(o);
+        let view = make_view(get(samples, i));
         views.push(view)
     }
     if timer.echo() {
@@ -526,7 +538,7 @@ fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
             "{}",
             timer.stamp("Made data structures.\n").unwrap().as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let selected_candidates_with_rngs: Vec<_> =
         rand::seq::index::sample(&mut rng, n_samples, n_samples)
@@ -543,7 +555,7 @@ fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
             "{}",
             timer.stamp("Selected all candidates.\n").unwrap().as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let selected_candidates_with_rngs: Vec<_> = pool.install(|| {
         selected_candidates_with_rngs
@@ -567,12 +579,12 @@ fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
                 .unwrap()
                 .as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let mut candidates = Vec::with_capacity(selected_candidates_with_rngs.len());
     for (z, rng) in selected_candidates_with_rngs {
-        if interrupted || r::check_user_interrupt() {
-            panic!("Caught user interrupt before main loop, so aborting.");
+        if interrupted || R::check_user_interrupt() {
+            stop!("Caught user interrupt before main loop, so aborting.");
         }
         let loss = expected_loss_from_samples(z.view(), &views, a, &pool);
         candidates.push((z, loss, rng));
@@ -593,11 +605,11 @@ fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
     });
     if !quiet {
         rprint!("\n");
-        r::flush_console();
+        R::flush_console();
     }
     if timer.echo() {
         rprint!("{}", timer.stamp("Sweetened bests.\n").unwrap().as_str());
-        r::flush_console();
+        R::flush_console();
     }
     bests.sort_unstable_by(|x, y| x.2.partial_cmp(&y.2).unwrap());
     let (best_z, _, best_loss, candidate_number, n_accepts, best_iteration, _) =
@@ -615,7 +627,7 @@ fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
             )
             .as_str()
         );
-        r::flush_console();
+        R::flush_console();
     }
     let columns_to_keep: Vec<usize> = best_z
         .axis_iter(Axis(1))
@@ -628,27 +640,29 @@ fn draws(samples: Rval, a: Rval, n_cores: Rval, quiet: Rval) -> Rval {
             }
         })
         .collect();
-    let (estimate, estimate_slice) = Rval::new_matrix_double(n_items, columns_to_keep.len(), pc);
+    let (estimate, estimate_slice) = RMatrix::new_double(n_items, columns_to_keep.len(), pc);
     columns_to_keep
         .iter()
         .enumerate()
         .for_each(|(j_new, j_old)| {
             matrix_copy_into_column(estimate_slice, n_items, j_new, best_z.column(*j_old).iter())
         });
-    let list = Rval::new_list(3, pc);
-    list.names_gets(rval!(["estimate", "expectedLoss", "secondsTotal",]));
-    list.set_list_element(0, estimate);
-    list.set_list_element(1, rval!(best_loss));
-    list.set_list_element(2, rval!(timer.total_as_secs_f64()));
+    let list = RList::new(3, pc);
+    let _ = list.names_gets(rstr!(["estimate", "expectedLoss", "secondsTotal",]));
+    let _ = list.set(0, estimate);
+    let _ = list.set(1, rvec!(best_loss));
+    let _ = list.set(2, rvec!(timer.total_as_secs_f64()));
     if timer.echo() {
         rprint!("{}", timer.stamp("Finalized results.\n").unwrap().as_str());
-        r::flush_console();
+        R::flush_console();
     }
     list
 }
 
 #[roxido]
-fn compute_expected_loss(z: Rval, samples: Rval, a: Rval, n_cores: Rval) -> Rval {
+fn compute_expected_loss(z: RObject, samples: RObject, a: RObject, n_cores: RObject) -> RObject {
+    let z = z.as_matrix_or_stop("Expected a matrix.");
+    let samples = samples.as_list_or_stop("Expected a list.");
     let a = a.as_f64();
     let n_cores = n_cores.as_usize();
     let pool = rayon::ThreadPoolBuilder::new()
@@ -658,13 +672,15 @@ fn compute_expected_loss(z: Rval, samples: Rval, a: Rval, n_cores: Rval) -> Rval
     let n_samples = samples.len();
     let mut views = Vec::with_capacity(n_samples);
     for i in 0..n_samples {
-        views.push(make_view(samples.get_list_element(i)));
+        views.push(make_view(get(samples, i)));
     }
-    rval!(expected_loss_from_samples(make_view(z), &views, a, &pool))
+    rvec!(expected_loss_from_samples(make_view(z), &views, a, &pool))
 }
 
 #[roxido]
-fn compute_loss(z1: Rval, z2: Rval, a: Rval) -> Rval {
+fn compute_loss(z1: RObject, z2: RObject, a: RObject) -> RObject {
+    let z1 = z1.as_matrix_or_stop("'z1' is not a matrix.");
+    let z2 = z2.as_matrix_or_stop("'z2' is not a matrix.");
     let a = a.as_f64();
     let loss = if z1.is_double() && z2.is_double() && z1.nrow() == z2.nrow() {
         match make_weight_matrix(make_view(z1), make_view(z2), a) {
@@ -672,14 +688,16 @@ fn compute_loss(z1: Rval, z2: Rval, a: Rval) -> Rval {
             None => 0.0,
         }
     } else {
-        panic!("Unsupported or inconsistent types for 'Z1' and 'Z2'.");
+        stop!("Unsupported or inconsistent types for 'Z1' and 'Z2'.");
     };
-    rval!(loss)
+    rvec!(loss)
 }
 
 #[roxido]
-fn compute_loss_permutations(z1: Rval, z2: Rval, a: Rval) -> Rval {
+fn compute_loss_permutations(z1: RObject, z2: RObject, a: RObject) -> RObject {
     use itertools::Itertools;
+    let z1 = z1.as_matrix_or_stop("'z1' is not a matrix.");
+    let z2 = z2.as_matrix_or_stop("'z2' is not a matrix.");
     let a = a.as_f64();
     let b = 2.0 - a;
     let loss = if z1.is_double() && z2.is_double() && z1.nrow() == z2.nrow() {
@@ -692,13 +710,12 @@ fn compute_loss_permutations(z1: Rval, z2: Rval, a: Rval) -> Rval {
             .permutations(k)
             .map(|permutation| {
                 let mut loss = 0.0;
-                for i in 0..k {
+                for (i, &j) in permutation.iter().enumerate().take(k) {
                     let c1 = if i >= v1.ncols() {
                         zero_view
                     } else {
                         v1.column(i)
                     };
-                    let j = permutation[i];
                     let c2 = if j >= v2.ncols() {
                         zero_view
                     } else {
@@ -722,13 +739,15 @@ fn compute_loss_permutations(z1: Rval, z2: Rval, a: Rval) -> Rval {
             .reduce(f64::min)
             .unwrap()
     } else {
-        panic!("Unsupported or inconsistent types for 'Z1' and 'Z2'.");
+        stop!("Unsupported or inconsistent types for 'Z1' and 'Z2'.");
     };
-    rval!(loss)
+    rvec!(loss)
 }
 
 #[roxido]
-fn compute_loss_augmented(z1: Rval, z2: Rval, a: Rval) -> Rval {
+fn compute_loss_augmented(z1: RObject, z2: RObject, a: RObject) -> RObject {
+    let z1 = z1.as_matrix_or_stop("'z1' is not a matrix.");
+    let z2 = z2.as_matrix_or_stop("'z2' is not a matrix.");
     let a = a.as_f64();
     let (loss, mut solution) = if z1.is_double() && z2.is_double() {
         match make_weight_matrix(make_view(z1), make_view(z2), a) {
@@ -739,7 +758,7 @@ fn compute_loss_augmented(z1: Rval, z2: Rval, a: Rval) -> Rval {
             None => (0.0, (vec![], vec![])),
         }
     } else {
-        panic!("Unsupported or inconsistent types for 'Z1' and 'Z2'.");
+        stop!("Unsupported or inconsistent types for 'Z1' and 'Z2'.");
     };
     for x in solution.0.iter_mut() {
         *x += 1;
@@ -747,11 +766,11 @@ fn compute_loss_augmented(z1: Rval, z2: Rval, a: Rval) -> Rval {
     for x in solution.1.iter_mut() {
         *x += 1;
     }
-    let list = Rval::new_list(3, pc);
-    list.names_gets(rval!(["loss", "permutation1", "permutation2"]));
-    list.set_list_element(0, rval!(loss));
-    list.set_list_element(1, Rval::try_new(&solution.1[..], pc).unwrap());
-    list.set_list_element(2, Rval::try_new(&solution.0[..], pc).unwrap());
+    let list = RList::new(3, pc);
+    let _ = list.names_gets(rstr!(["loss", "permutation1", "permutation2"]));
+    let _ = list.set(0, rvec!(loss));
+    let _ = list.set(1, RVector::try_allocate(&solution.1[..], pc).unwrap());
+    let _ = list.set(2, RVector::try_allocate(&solution.0[..], pc).unwrap());
     list
 }
 
@@ -765,8 +784,12 @@ fn matrix_copy_into_column<'a>(
     subslice.iter_mut().zip(iter).for_each(|(x, y)| *x = *y);
 }
 
-fn make_view(z: Rval) -> ArrayView2<'static, f64> {
-    unsafe { ArrayView::from_shape_ptr((z.nrow(), z.ncol()).f(), z.try_into().unwrap()) }
+fn make_view(z: RMatrix) -> ArrayView2<'static, f64> {
+    let slice = match z.slice_double() {
+        Ok(slice) => slice,
+        Err(_) => stop!("Not of storage mode 'double'."),
+    };
+    unsafe { ArrayView::from_shape_ptr((z.nrow(), z.ncol()).f(), slice.as_ptr()) }
 }
 
 fn make_weight_matrices(
@@ -808,12 +831,10 @@ fn flip_bit(
                 } else {
                     -b
                 }
+            } else if bit_in_sample == 0.0 {
+                -a
             } else {
-                if bit_in_sample == 0.0 {
-                    -a
-                } else {
-                    b
-                }
+                b
             };
         }
     });
@@ -848,12 +869,10 @@ fn update_w(
             } else {
                 -b
             }
+        } else if bit_in_sample == 0.0 {
+            -a
         } else {
-            if bit_in_sample == 0.0 {
-                -a
-            } else {
-                b
-            }
+            b
         };
     }
 }
